<?php


namespace Edfapay\Gateway\Controller\Url;

use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;

/**
 * 
 * Class Index
 * @package Edfapay\Gateway\Controller\Ipn
 */
class Response extends \Edfapay\Gateway\Controller\AbstractAction implements CsrfAwareActionInterface
{

    /**
     * Instantiate IPN model and pass IPN request to it
     *
     * @return void
     */
    public function execute()
    {
        $logger = $this->getLogger();
        $request = $this->getRequest();
        $objectManager = $this->getObjectManager();

        $logger->debug('Response Controller Started');

        if (!$request->isPost()) {
            return;
        }

        try {
            $ipn = $objectManager->create("Edfapay\\Gateway\\Model\\Ipn\\EdfapayGatewayIpn");
            $data = $request->getPostValue();
            $responseBody = $ipn->handleEdfapayGatewayNotification($data);
            $logger->debug("Response Body");
            $logger->debug(print_r($responseBody, true));

            $response = $this->getResponse()
                ->setHeader('Content-type', 'text/html')
                ->setBody($responseBody['body'])
                ->setHttpResponseCode($responseBody['code']);

            $response->sendResponse();
            $logger->debug('Response Controller Ended');
        } catch (\Exception $e) {
            $logger->critical($e);
            $this->getResponse()->setHttpResponseCode(500);
        }
    }


    /**
     * @inheritDoc
     */
    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }

    /**
     * @inheritDoc
     */
    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }
}
