<?php


namespace Edfapay\Gateway\Model\Ipn;

/**
 * Checkout Method IPN Handler Class
 * Class CheckoutIpn
 * @package Edfapay\Gateway\Model\Ipn
 */
class EdfapayGatewayIpn extends \Edfapay\Gateway\Model\Ipn\AbstractIpn
{
    /**
     * @return string
     */
    protected function getPaymentMethodCode()
    {
        return \Edfapay\Gateway\Model\Method\Checkout::CODE;
    }

    /**
     * Update Pending Transactions and Order Status
     * @param \stdClass $responseObject
     * @throws \Exception
     */
    protected function processNotification($responseObject)
    {
        $payment = $this->getPayment();
        $helper = $this->getModuleHelper();

        // Determine order details based on the order type
        if (isset($responseObject['order_number'])) {
            $isApplePayOrder = true;
            $payment_trans_id = $responseObject['id'];
            $payment_order_id = $responseObject['order_number'];
            $payment_transaction_amount = $responseObject['order_amount'];
        } else {
            $isApplePayOrder = false;
            $payment_trans_id = $responseObject['trans_id'];
            $payment_order_id = $responseObject['order_id'];
            $payment_transaction_amount = $responseObject['amount'];
        }

        // Update transaction additional info
        $helper->updateTransactionAdditionalInfo($payment_trans_id, $responseObject, true);

        // Update payment and order details
        $payment
            ->setLastTransId($payment_trans_id)
            ->setTransactionId($payment_trans_id)
            ->setParentTransactionId($payment_order_id ?: null)
            ->setIsTransactionPending($this->getShouldSetCurrentTranPending($responseObject))
            ->setShouldCloseParentTransaction(true)
            ->setIsTransactionClosed(false)
            ->setPreparedMessage($this->createIpnComment('Successful payment'))
            ->resetTransactionAdditionalInfo();

        // Set payment transaction additional info
        $helper->setPaymentTransactionAdditionalInfo($payment, $responseObject);
        $payment->registerCaptureNotification($payment_transaction_amount);
        $payment->save();

        // Set order status
        $helper->setOrderStatusByState($this->getOrder(), \Magento\Sales\Model\Order::STATE_PROCESSING);
        $this->getOrder()->save();
    }
}
