<?php

namespace Edfapay\Gateway\Model\Method;

use Klarna\Core\Model\LogFactory;
use Magecrafts\Log\Model\Log;
use Magento\Checkout\Model\Session;
use Magento\Framework\Api\AttributeValueFactory;
use Magento\Framework\Api\ExtensionAttributesFactory;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\HTTP\Client\Curl;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\Registry;
use Magento\Framework\UrlInterface;
use Magento\Payment\Helper\Data;
use Magento\Payment\Model\Method\AbstractMethod;
use Magento\Store\Model\StoreManagerInterface;
use Edfapay\Gateway\Model\Traits\Logger;
use Edfapay\Gateway\Model\Traits\OnlinePaymentMethod;
use Zend_Log;

/**
 * Checkout Payment Method Model Class
 * Class Checkout
 * @package Edfapay\Gateway\Model\Method
 */
class Checkout extends AbstractMethod
{
    use OnlinePaymentMethod;
    use Logger;

    const CODE = 'edfapay_checkout';
    /**
     * Checkout Method Code
     */
    protected $_code = self::CODE;

    protected $_canOrder = true;
    protected $_isGateway = true;
    protected $_canCapture = true;
    protected $_canCapturePartial = true;
    protected $_canRefund = true;
    protected $_canCancelInvoice = true;
    protected $_canVoid = true;
    protected $_canRefundInvoicePartial = true;
    protected $_canAuthorize = true;
    protected $_isInitializeNeeded = false;
    protected Curl $curl;
    private UrlInterface $url;

    /**
     * @param Curl $curl
     * @param Context $context
     * @param \Magento\Framework\App\Action\Context $actionContext
     * @param Registry $registry
     * @param ExtensionAttributesFactory $extensionFactory
     * @param AttributeValueFactory $customAttributeFactory
     * @param Data $paymentData
     * @param ScopeConfigInterface $scopeConfig
     * @param \Magento\Payment\Model\Method\Logger $logger
     * @param StoreManagerInterface $storeManager
     * @param Session $checkoutSession
     * @param \Edfapay\Gateway\Helper\Data $moduleHelper
     * @param AbstractResource|null $resource
     * @param AbstractDb|null $resourceCollection
     * @param array $data
     * @throws LocalizedException
     */
    public function __construct(
        UrlInterface $url,
        Curl                                  $curl,
        Context                               $context,
        \Magento\Framework\App\Action\Context $actionContext,
        Registry                              $registry,
        ExtensionAttributesFactory            $extensionFactory,
        AttributeValueFactory                 $customAttributeFactory,
        Data                                  $paymentData,
        ScopeConfigInterface                  $scopeConfig,
        \Magento\Payment\Model\Method\Logger  $logger,
        StoreManagerInterface                 $storeManager,
        Session                               $checkoutSession,
        \Edfapay\Gateway\Helper\Data              $moduleHelper,
        AbstractResource                      $resource = null,
        AbstractDb                            $resourceCollection = null,
        array                                 $data = []
    ) {
        parent::__construct(
            $context,
            $registry,
            $extensionFactory,
            $customAttributeFactory,
            $paymentData,
            $scopeConfig,
            $logger,
            $resource,
            $resourceCollection,
            $data
        );
        $this->url = $url;
        $this->curl = $curl;
        $this->_actionContext = $actionContext;
        $this->_storeManager = $storeManager;
        $this->_checkoutSession = $checkoutSession;
        $this->_moduleHelper = $moduleHelper;
        $this->_logger = $this->_initLogger();
        $this->_configHelper =
            $this->getModuleHelper()->getMethodConfig(
                $this->getCode()
            );
    }

    /**
     * @return \Zend\Log\Logger|Zend_Log
     */
    protected function getLogger()
    {
        return $this->_logger;
    }

    /**
     * Get Default Payment Action On Payment Complete Action
     * @return string
     */
    public function getConfigPaymentAction()
    {
        return AbstractMethod::ACTION_ORDER;
    }

    /**
     * Get Available Checkout Transaction Types
     * @return array
     */
    public function getCheckoutTransactionTypes()
    {
        return $this->getConfigHelper()->getTransactionTypes();
    }

    /**
     * Create a Web-Payment Form Instance
     *
     * @param $data
     * @return string
     * @throws LocalizedException
     */
    protected function checkout($data)
    {
        $merchantId = $this->_scopeConfig->getValue('payment/edfapay_checkout/shop_id', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $newData = [
            'action' => 'SALE',
            'order_id' => $data['order']['increment_id'],
            'edfa_merchant_id' => $merchantId,
            'order_amount' => number_format($data['order']['amount'], 2, '.', ''),
            'order_currency' => $data['order']['currency'],
            'order_description' => $data['order']['description'],
            'req_token' => 'N',
            'payer_first_name' => $data['order']['billing']->getFirstname(),
            'payer_last_name' => $data['order']['billing']->getLastname(),
            'payer_address' => 'this is address',
            'payer_country' => $data['order']['billing']->getCountryId(),
            'payer_city' => $data['order']['billing']->getCity(),
            'payer_zip' => $data['order']['billing']->getPostcode(),
            'payer_email' => $data['order']['customer']['email'],
            'payer_phone' => $data['order']['billing']->getTelephone(),
            'payer_ip' => '176.44.76.222',
            'term_url_3ds' => $this->url->getUrl('edfapay/checkout/redirect'),
            'auth' => 'N',
            'recurring_init' => 'N',
            'hash' => $this->getHashValue($data)
        ];
        $response = $this->formrequest($newData);
        return $response;
    }

    public function getHashValue($data)
    {
        $merchantPass = $this->_scopeConfig->getValue('payment/edfapay_checkout/shop_key', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        // var to_md5 = order_number + order_amount + order_currency + order_description + merchant_pass;
        $str_to_hash = $data['order']['increment_id'] .
            number_format($data['order']['amount'], 2, '.', '') .
            $data['order']['currency'] . $data['order']['description'] .
            $merchantPass;
        return sha1(md5(strtoupper($str_to_hash)));
    }

    /**
     * @param $data
     * @return string
     */
    public function formrequest($data)
    {
        $url = $this->getConfigHelper()->getDomainCheckout();
        $this->curl->get($url);
        $this->curl->post($url, $data);
        return $this->curl->getBody();
    }

    /**
     * Order Payment
     * @param \Magento\Payment\Model\InfoInterface $payment
     * @param float $amount
     * @return $this
     * @throws LocalizedException
     */

    public function order(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        $order = $payment->getOrder();
        $orderId = ltrim(
            $order->getIncrementId(),
            '0'
        );
        $ord_id = $this->getModuleHelper()->genTransactionId($order->getIncrementId());
        $data = [
            'order' => [
                'increment_id' => $ord_id,
                'currency' => $order->getBaseCurrencyCode(),
                'amount' => $amount,
                'ip_address' => $order->getData('remote_ip'),
                //'usage' => $this->getModuleHelper()->buildOrderUsage(),
                //                'description' => __('Order # %1 payment', $orderId),
                'description' => 'Order # ' . $orderId . ' payment',
                'customer' => [
                    'email' => $this->getCheckoutSession()->getQuote()->getCustomerEmail(),
                ],
                'billing' =>
                $order->getBillingAddress(),
                'shipping' =>
                $order->getShippingAddress()
            ],
        ];

        // $this->getConfigHelper()->initGatewayClient();//init api


        try {

            $responseObject = $this->checkout($data);
            $responseArr = json_decode((string)$responseObject, true);
            if (!isset($responseArr['redirect_url'])) {
                $this->_logger->debug(print_r($responseArr, true));
            }
            //return $this->_redirect->redirect($this->_response, $responseArr['redirect_url']);
            $payment->setTransactionId($ord_id);
            $payment->setIsTransactionPending(true);
            $payment->setIsTransactionClosed(false);
            $this->getModuleHelper()->setPaymentTransactionAdditionalInfo(
                $payment,
                $responseArr
            );
            $payment->setTransactionAdditionalInfo(\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS, $responseArr);
            $this->getCheckoutSession()->setEdfapayGatewayCheckoutRedirectUrl($responseArr['redirect_url']);

            //$this->_writeDebugData();
            return $this;
        } catch (\Exception $e) {
            $this->_logger->debug('exceptionORDER', $e->getMessage());
            $this->getCheckoutSession()->setEdfapayGatewayLastCheckoutError(
                $e->getMessage()
            );
            //$this->_writeDebugData();
            //$this->getModuleHelper()->maskException($e);
        }
    }

    /**
     * Payment Capturing
     * @param \Magento\Payment\Model\InfoInterface $payment
     * @param float $amount
     * @return $this
     * @throws \Magento\Framework\Webapi\Exception
     */
    public function capture(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        // /** @var \Magento\Sales\Model\Order $order */
        // $order = $payment->getOrder();

        // $this->_addDebugData('capture_process', 'Capture transaction for order #' . $order->getIncrementId());

        // $authTransaction = $this->getModuleHelper()->lookUpAuthorizationTransaction(
        //     $payment
        // );

        // if (!isset($authTransaction)) {
        //     $errorMessage = __('Capture transaction for order # %1 cannot be finished (No Authorize Transaction exists)',
        //         $order->getIncrementId()
        //     );

        //     $this->_addDebugData('capture_error', $errorMessage);
        //     $this->_writeDebugData();

        //     $this->getModuleHelper()->throwWebApiException(
        //         $errorMessage
        //     );
        // }

        // try {
        //     $this->doCapture($payment, $amount, $authTransaction);
        // } catch (\Exception $e) {
        //     $this->_addDebugData('exception', $e->getMessage());
        //     $this->_writeDebugData();
        //     $this->getModuleHelper()->maskException($e);
        // }
        // $this->_writeDebugData();

        return $this;
    }

    /**
     * Payment refund
     *
     * @param \Magento\Payment\Model\InfoInterface $payment
     * @param float $amount
     * @return $this
     * @throws \Magento\Framework\Webapi\Exception
     */
    public function refund(\Magento\Payment\Model\InfoInterface $payment, $amount)
    {
        /** @var \Magento\Sales\Model\Order $order */
        $order = $payment->getOrder();
        $this->_logger->debug('refund_process', 'Refund transaction for order #' . $order->getIncrementId());

        $captureTransaction = $this->getModuleHelper()->lookUpCaptureTransaction($payment);

        if (!$captureTransaction) {
            $errorMessage = __('Refund transaction for order # %1 cannot be finished (No Capture Transaction exists)', $order->getIncrementId());
            $this->_handleRefundError($errorMessage);
        }

        try {
            $this->doRefund($payment, $amount, $captureTransaction);
        } catch (\Exception $e) {
            $this->_handleRefundError($e->getMessage());
            $this->getModuleHelper()->maskException($e);
        }

        return $this;
    }

    /**
     * Handle refund error
     *
     * @param string $errorMessage
     */
    private function _handleRefundError($errorMessage)
    {
        $this->_addDebugData('refund_error', $errorMessage);
        $this->_writeDebugData();
        $this->getMessageManager()->addError($errorMessage);
        $this->getModuleHelper()->throwWebApiException($errorMessage);
        $this->_writeDebugData();
    }


    /**
     * Payment Cancel
     * @param \Magento\Payment\Model\InfoInterface $payment
     * @return $this
     */
    public function cancel(\Magento\Payment\Model\InfoInterface $payment)
    {
        $this->void($payment);
        return $this;
    }

    /**
     * Void Payment
     * @param \Magento\Payment\Model\InfoInterface $payment
     * @return $this
     * @throws \Magento\Framework\Webapi\Exception
     */
    public function void(\Magento\Payment\Model\InfoInterface $payment)
    {
        // /** @var \Magento\Sales\Model\Order $order */

        // $order = $payment->getOrder();

        // $this->_addDebugData('void_process', 'Void transaction for order #' . $order->getIncrementId());

        // $referenceTransaction = $this->getModuleHelper()->lookUpVoidReferenceTransaction(
        //     $payment
        // );

        // if ($referenceTransaction->getTxnType() == \Magento\Sales\Model\Order\Payment\Transaction::TYPE_AUTH) {
        //     $authTransaction = $referenceTransaction;
        // } else {
        //     $authTransaction = $this->getModuleHelper()->lookUpAuthorizationTransaction(
        //         $payment
        //     );
        // }

        // if (!isset($authTransaction) || !isset($referenceTransaction)) {
        //     $errorMessage = __('Void transaction for order # %1 cannot be finished (No Authorize / Capture Transaction exists)',
        //                     $order->getIncrementId()
        //     );
        //     $this->_addDebugData('void_error', $errorMessage);
        //     $this->_writeDebugData();

        //     $this->getModuleHelper()->throwWebApiException($errorMessage);
        // }

        // try {
        //     $this->doVoid($payment, $authTransaction, $referenceTransaction);
        // } catch (\Exception $e) {
        //     $this->_addDebugData('exception', $e->getMessage());
        //     $this->_writeDebugData();
        //     $this->getModuleHelper()->maskException($e);
        // }
        // $this->_writeDebugData();

        return $this;
    }

    /**
     * Determines method's availability based on config data and quote amount
     *
     * @param \Magento\Quote\Api\Data\CartInterface|null $quote
     * @return bool
     */
    public function isAvailable(\Magento\Quote\Api\Data\CartInterface $quote = null)
    {
        return parent::isAvailable($quote) &&
            $this->getConfigHelper()->isMethodAvailable();
    }

    // /**
    //  * Checks base currency against the allowed currency
    //  *
    //  * @param string $currencyCode
    //  * @return bool
    //  */
    // public function canUseForCurrency($currencyCode)
    // {
    //     return $this->getModuleHelper()->isCurrencyAllowed(
    //         $this->getCode(),
    //         $currencyCode
    //     );
    // }
}
